@@ -89,14 +89,9 @@ class Function(_instance_base.SimpleValue):
     return self._match_args_sequentially(node, args, alias_map, match_all_views)
 
   def _is_complex_generic_call(self, args):
-    signatures = function.get_signatures(self)
-    # pytype: disable=attribute-error
-    if _isinstance(self, "SignedFunction") and self.signature not in signatures:
-      # This happens for overloads. While overloads are not callable, we do call
-      # match_args on them to catch bad defaults.
-      signatures.append(self.signature)
-    # pytype: enable=attribute-error
-    for sig in signatures:
+    if _isinstance(self, "SignedFunction"):
+      return False
+    for sig in function.get_signatures(self):
       parameter_typevar_count = 0
       for name, t in sig.annotations.items():
         stack = [t]
@@ -288,9 +283,9 @@ class BoundFunction(_base.BaseValue):
     if self.underlying.should_replace_self_annot():
       self.replace_self_annot = abstract_utils.get_generic_type(inst)
     if isinstance(inst, _instance_base.SimpleValue):
-      self.alias_map = inst.instance_type_parameters.uf
+      self.alias_map = inst.instance_type_parameters.aliases
     elif isinstance(inst, _typing.TypeParameterInstance):
-      self.alias_map = inst.instance.instance_type_parameters.uf
+      self.alias_map = inst.instance.instance_type_parameters.aliases
     else:
       self.alias_map = None
 
@@ -306,7 +301,7 @@ class BoundFunction(_base.BaseValue):
     return self._callself
 
   def call(self, node, func, args, alias_map=None):
-    if abstract_utils.func_name_is_class_init(self.name):
+    if self.name.endswith(".__init__"):
       self.ctx.callself_stack.append(self._callself)
     # The "self" parameter is automatically added to the list of arguments, but
     # only if the function actually takes any arguments.
@@ -329,7 +324,7 @@ class BoundFunction(_base.BaseValue):
         e.name = f"{self._callself.data[0].name}.{e.name}"
       raise
     finally:
-      if abstract_utils.func_name_is_class_init(self.name):
+      if self.name.endswith(".__init__"):
         self.ctx.callself_stack.pop()
     return node, ret
 
