@@ -46,32 +46,12 @@ def _matches_async_generator(type_obj):
   return _matches_generator_helper(type_obj, allowed_types)
 
 
-def _hash_dict(vardict, names):
-  """Hash a dictionary.
-
-  This contains the keys and the full hashes of the data in the values.
-
-  Arguments:
-    vardict: A dictionary mapping str to Variable.
-    names: If this is non-None, the snapshot will include only those
-      dictionary entries whose keys appear in names.
-
-  Returns:
-    A hash of the dictionary.
-  """
-  if names is not None:
-    vardict = {name: vardict[name] for name in names.intersection(vardict)}
-  m = hashlib.md5()
-  for name, var in sorted(vardict.items()):
-    m.update(str(name).encode("utf-8"))
-    for value in var.bindings:
-      m.update(value.data.get_fullhash())
-  return m.digest()
-
-
 def _hash_all_dicts(*hash_args):
   """Convenience method for hashing a sequence of dicts."""
-  return hashlib.md5(b"".join(_hash_dict(*args) for args in hash_args)).digest()
+  components = (abstract_utils.get_dict_fullhash_component(d, names=n)
+                for d, n in hash_args)
+  return hashlib.md5(
+      b"".join(str(hash(c)).encode("utf-8") for c in components)).digest()
 
 
 def _check_classes(var, check):
@@ -236,8 +216,7 @@ class SignedFunction(_function_base.Function):
     return subst
 
   def _match_args_sequentially(self, node, args, alias_map, match_all_views):
-    matcher = self.ctx.matcher(node)
-    substs = None
+    args_to_match = []
     for name, arg, formal in self.signature.iter_args(args):
       if formal is None:
         continue
@@ -245,19 +224,14 @@ class SignedFunction(_function_base.Function):
         # The annotation is Tuple or Dict, but the passed arg only has to be
         # Iterable or Mapping.
         formal = self.ctx.convert.widen_type(formal)
-      match_result = matcher.compute_matches(
-          arg, formal, name, match_all_views)
-      if not match_result.success:
-        raise function.WrongArgTypes(
-            self.signature, args, self.ctx,
-            bad_param=match_result.bad_matches[0].expected)
-      cur_substs = [m.subst for m in match_result.good_matches]
-      if any(cur_substs):
-        assert substs is None
-        substs = cur_substs
-    if not substs:
-      substs = [matcher.default_match().subst]
-    return substs
+      args_to_match.append(function.Arg(name, arg, formal))
+    matcher = self.ctx.matcher(node)
+    try:
+      matches = matcher.compute_matches(
+          args_to_match, match_all_views, alias_map)
+    except matcher.MatchError as e:
+      raise function.WrongArgTypes(self.signature, args, self.ctx, e.bad_type)
+    return [m.subst for m in matches]
 
   def get_first_opcode(self):
     return None
@@ -642,12 +616,29 @@ class InterpreterFunction(SignedFunction):
         [self.ctx.convert.unsolvable, self.ctx.convert.none], [], node)
     return function.Args(posargs=(args.posargs[0], arg1, arg2, arg3))
 
+  def _hash_call(self, callargs, frame):
+    # Note that we ignore caching in __init__ calls, so that attributes are
+    # set correctly.
+    if (self.ctx.options.skip_repeat_calls and
+        ("self" not in callargs or not self.ctx.callself_stack or
+         callargs["self"].data != self.ctx.callself_stack[-1].data)):
+      callkey = _hash_all_dicts(
+          (callargs, None),
+          (frame.f_globals.members, set(self.code.co_names)),
+          (frame.f_locals.members,
+           set(frame.f_locals.members) - set(self.code.co_varnames)))
+    else:
+      # Make the callkey the number of times this function has been called so
+      # that no call has the same key as a previous one.
+      callkey = len(self._call_cache)
+    return callkey
+
   def call(self, node, func, args, alias_map=None, new_locals=False,
            frame_substs=()):
     if self.is_overload:
       raise function.NotCallable(self)
     if (self.ctx.vm.is_at_maximum_depth() and
-        not abstract_utils.func_name_is_class_init(self.name)):
+        not self.name.endswith(".__init__")):
       log.info("Maximum depth reached. Not analyzing %r", self.name)
       self._set_callself_maybe_missing_members()
       if (self.signature.has_return_annotation and
@@ -741,18 +732,9 @@ class InterpreterFunction(SignedFunction):
       frame.allowed_returns = annotations.get("return",
                                               self.ctx.convert.unsolvable)
       frame.check_return = check_return
-    if self.ctx.options.skip_repeat_calls:
-      callkey = _hash_all_dicts(
-          (callargs, None),
-          (frame.f_globals.members, set(self.code.co_names)),
-          (frame.f_locals.members,
-           set(frame.f_locals.members) - set(self.code.co_varnames)))
-    else:
-      # Make the callkey the number of times this function has been called so
-      # that no call has the same key as a previous one.
-      callkey = len(self._call_cache)
-    if callkey in self._call_cache:
-      old_ret, old_remaining_depth = self._call_cache[callkey]
+    callkey_pre = self._hash_call(callargs, frame)
+    if callkey_pre in self._call_cache:
+      old_ret, old_remaining_depth = self._call_cache[callkey_pre]
       # Optimization: This function has already been called, with the same
       # environment and arguments, so recycle the old return value.
       # We would want to skip this optimization and reanalyze the call if we can
@@ -808,7 +790,9 @@ class InterpreterFunction(SignedFunction):
         ret = _instances.Coroutine(self.ctx, ret, node2).to_variable(node2)
       node_after_call = node2
     self._inner_cls_check(frame)
-    self._call_cache[callkey] = ret, self.ctx.vm.remaining_depth()
+    # Recompute the calllkey so that side effects are taken into account.
+    callkey_post = self._hash_call(callargs, frame)
+    self._call_cache[callkey_post] = ret, self.ctx.vm.remaining_depth()
     if self._store_call_records or self.ctx.store_all_calls:
       self._call_records.append((callargs, ret, node_after_call))
     self.last_frame = frame
@@ -882,8 +866,7 @@ class InterpreterFunction(SignedFunction):
     return self.code.has_varkeywords()
 
   def property_get(self, callself, is_class=False):
-    if (abstract_utils.func_name_is_class_init(self.name) and
-        self.signature.param_names):
+    if self.name.endswith(".__init__") and self.signature.param_names:
       self_name = self.signature.param_names[0]
       # If `_has_self_annot` is True, then we've intentionally temporarily
       # annotated `self`; otherwise, a `self` annotation is illegal.
