@@ -3,7 +3,7 @@ import collections
 import contextlib
 import dataclasses
 import logging
-from typing import Dict, List, Optional
+from typing import Any, Dict, Generic, List, Optional, Tuple, TypeVar
 
 from pytype import datatypes
 from pytype import special_builtins
@@ -27,7 +27,9 @@ _COMPATIBLE_BUILTINS = [
     for compatible_builtin, builtin in pep484.COMPAT_ITEMS
 ]
 
-_SubstType = Dict[str, cfg.Variable]
+_SubstType = datatypes.AliasingDict[str, cfg.Variable]
+_ViewType = Dict[cfg.Variable, cfg.Binding]
+_T = TypeVar("_T")
 
 
 def _is_callback_protocol(typ):
@@ -88,15 +90,24 @@ class ErrorDetails:
 class GoodMatch:
   """A correct type/actual value match."""
 
-  view: Dict[cfg.Variable, cfg.Binding]
-  subst: Dict[str, cfg.Variable]
+  view: _ViewType
+  subst: _SubstType
+
+  @classmethod
+  def default(cls):
+    return cls({}, datatypes.HashableDict())
+
+  @classmethod
+  def merge(cls, old_match, new_match, combined_subst):
+    return cls({**old_match.view, **new_match.view},
+               datatypes.HashableDict(combined_subst))
 
 
 @dataclasses.dataclass(eq=True, frozen=True)
 class BadMatch:
   """An expected type/actual value mismatch."""
 
-  view: Dict[cfg.Variable, cfg.Binding]
+  view: _ViewType
   expected: abstract_utils.BadType
   actual: cfg.Variable
 
@@ -111,22 +122,68 @@ class BadMatch:
 
 @dataclasses.dataclass(eq=True, frozen=True)
 class MatchResult:
-  """The result of a compute_match call."""
+  """The result of a compute_one_match call."""
 
   success: bool
   good_matches: List[GoodMatch]
   bad_matches: List[BadMatch]
 
 
+class _UniqueSubsts(Generic[_T]):
+  """A collection of substs that discards duplicates.
+
+  Each subst has a bit of data associated to it, and calling unique() returns
+  unique substs and associated data.
+  """
+
+  def __init__(self):
+    self._data: List[Tuple[Dict[str, Any], _SubstType, _T]] = []
+
+  def insert(self, subst, data: _T):
+    """Insert a subst with associated data."""
+    subst_key = {k: {v.get_type_key() for v in var.data}
+                 for k, var in subst.items()}
+    data_item = (subst_key, subst, data)
+    for i, (prev_subst_key, *_) in enumerate(self._data):
+      if all(k in prev_subst_key and subst_key[k] <= prev_subst_key[k]
+             for k in subst_key):
+        # A previous substitution is a superset of this one, so we do not need
+        # to keep this one.
+        break
+      if all(k in subst_key and prev_subst_key[k] <= subst_key[k]
+             for k in prev_subst_key):
+        # This substitution is a superset of a previous one, so we replace the
+        # previous subst with this one.
+        self._data[i] = data_item
+        break
+    else:
+      self._data.append(data_item)
+
+  def unique(self) -> List[Tuple[_SubstType, _T]]:
+    return [d[1:] for d in self._data]
+
+
 class AbstractMatcher(utils.ContextWeakrefMixin):
   """Matcher for abstract values."""
 
+  # This class is nested inside AbstractMatcher because matcher.py can't be
+  # imported in many of the places that the matcher is used.
+  class MatchError(Exception):
+
+    def __init__(self, bad_type: abstract_utils.BadType, *args, **kwargs):
+      self.bad_type = bad_type
+      super().__init__(bad_type, *args, **kwargs)
+
   def __init__(self, node, ctx):
     super().__init__(ctx)
     self._node = node
     self._protocol_cache = set()
-    self._recursive_annots_cache = set()
+    # Map from (actual value, expected recursive type) pairs to whether matching
+    # the value against the type succeeds.
+    self._recursive_annots_cache: Dict[
+        Tuple[abstract.BaseValue, abstract.BaseValue], bool] = {}
     self._error_subst = None
+    self._seen_type_params = set()
     self._reset_errors()
 
   def _reset_errors(self):
@@ -167,6 +224,8 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
             self._node, expected, [self._error_subst or {}]),
         error_details=self._error_details())
 
+  # TODO(b/228241343): Delete this method once all usages have been moved to
+  # compute_matches.
   def compute_subst(self, formal_args, arg_dict, view, alias_map=None):
     """Compute information about type parameters using one-way unification.
 
@@ -188,10 +247,8 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     if not arg_dict:
       # A call with no arguments always succeeds.
       assert not formal_args
-      return datatypes.AliasingDict(), None
-    subst = datatypes.AliasingDict()
-    if alias_map:
-      subst.uf = alias_map
+      return datatypes.HashableDict(), None
+    subst = datatypes.AliasingDict(aliases=alias_map)
     self._error_subst = None
     self_subst = None
     for name, formal in formal_args:
@@ -212,14 +269,51 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
           subst[name] = value
     return datatypes.HashableDict(subst), None
 
-  # This is a staticmethod rather than a module-level function because
-  # matcher.py can't be imported in many of the places that the matcher is used.
-  @staticmethod
-  def default_match() -> GoodMatch:
-    return GoodMatch({}, datatypes.HashableDict())
-
   def compute_matches(
-      self, var, other_type, name=None, match_all_views=True) -> MatchResult:
+      self, args: List[function.Arg], match_all_views: bool,
+      alias_map: Optional[datatypes.UnionFind] = None) -> List[GoodMatch]:
+    """Compute information about type parameters using one-way unification.
+
+    Given the arguments of a function call, try to find substitutions that match
+    them against their expected types.
+
+    Args:
+      args: A sequence of function arguments.
+      match_all_views: If True, every possible match must succeed for the
+        overall match to be considered a success. Otherwise, the overall match
+        succeeds as long as at least one possible match succeeds.
+      alias_map: Optionally, a datatypes.UnionFind, which stores all the type
+        renaming information, mapping of type parameter name to its
+        representative.
+    Returns:
+      A sequence of GoodMatch results containing the computed substitutions.
+    Raises:
+      MatchError: if any of the arguments does not match its expected type.
+    """
+    matches = None
+    has_self = args and args[0].name == "self"
+    for arg in args:
+      match_result = self.compute_one_match(
+          arg.value, arg.typ, arg.name, match_all_views, alias_map)
+      if not match_result.success:
+        if matches:
+          if self._error_subst:
+            self._error_subst = self._merge_substs(
+                matches[0].subst, [self._error_subst])
+          else:
+            self._error_subst = matches[0].subst
+          bad_param = self._get_bad_type(arg.name, arg.typ)
+        else:
+          bad_param = match_result.bad_matches[0].expected
+        raise self.MatchError(bad_param)
+      if any(m.subst for m in match_result.good_matches):
+        matches = self._merge_matches(
+            arg.name, arg.typ, matches, match_result.good_matches, has_self)
+    return matches if matches else [GoodMatch.default()]
+
+  def compute_one_match(
+      self, var, other_type, name=None, match_all_views=True, alias_map=None
+  ) -> MatchResult:
     """Match a Variable against a type.
 
     Args:
@@ -229,11 +323,14 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       match_all_views: If True, every possible match must succeed for the
         overall match to be considered a success. Otherwise, the overall match
         succeeds as long as at least one possible match succeeds.
+      alias_map: Optionally, a datatypes.UnionFind, which stores all the type
+        renaming information, mapping of type parameter name to its
+        representative.
     Returns:
       The match result.
     """
     bad_matches = []
-    good_matches = []
+    good_matches = _UniqueSubsts[_ViewType]()
     views = abstract_utils.get_views([var], self._node)
     skip_future = None
     while True:
@@ -241,7 +338,8 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         view = views.send(skip_future)
       except StopIteration:
         break
-      subst = self.match_var_against_type(var, other_type, {}, view)
+      subst = datatypes.AliasingDict(aliases=alias_map)
+      subst = self.match_var_against_type(var, other_type, subst, view)
       if subst is None:
         if self._node.HasCombination(list(view.values())):
           bad_matches.append(BadMatch(
@@ -253,7 +351,9 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         skip_future = False
       else:
         skip_future = True
-        good_matches.append(GoodMatch(view, datatypes.HashableDict(subst)))
+        good_matches.insert(subst, view)
+    good_matches = [GoodMatch(view, datatypes.HashableDict(subst))
+                    for subst, view in good_matches.unique()]
     if not bad_matches:
       success = True
     elif match_all_views or self.ctx.options.strict_parameter_checks:
@@ -317,6 +417,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         right_side_options = [other_type]
       for right in right_side_options:
         if isinstance(right, abstract.TypeParameter):
+          self._seen_type_params.add(right)
           # If we have a union like "K or V" and we match both against
           # nothing, that will fill in both K and V.
           if right.full_name not in subst:
@@ -353,7 +454,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
 
   def _match_value_against_type(
       self, value: cfg.Binding, other_type: abstract.BaseValue,
-      subst: _SubstType, view: Dict[cfg.Variable, cfg.Binding]
+      subst: _SubstType, view: _ViewType
   ) -> Optional[_SubstType]:
     """One-way unify value into pytd type given a substitution.
 
@@ -373,12 +474,25 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     other_type = abstract_utils.unwrap_final(other_type)
 
     # Make sure we don't recurse infinitely when matching recursive types.
-    if abstract_utils.is_recursive_annotation(other_type):
+    is_recursive = abstract_utils.is_recursive_annotation(other_type)
+    if is_recursive:
       key = (left, other_type)
       if key in self._recursive_annots_cache:
-        return subst
-      self._recursive_annots_cache.add(key)
+        return subst if self._recursive_annots_cache[key] else None
+      self._recursive_annots_cache[key] = True
 
+    subst = self._match_nonfinal_value_against_type(
+        left, value, other_type, subst, view)
+    if is_recursive:
+      self._recursive_annots_cache[key] = subst is not None
+    return subst
+
+  def _match_nonfinal_value_against_type(
+      self, left: abstract.BaseValue, value: cfg.Binding,
+      other_type: abstract.BaseValue, subst: _SubstType,
+      view: _ViewType
+  ) -> Optional[_SubstType]:
+    """Match after unwrapping any `Final` annotations."""
     if left.formal:
       # 'left' contains a TypeParameter. The code under analysis is likely doing
       # some sort of runtime processing of type annotations. We replace all type
@@ -396,6 +510,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
                                    function.Signature)) or
         left.instance is abstract_utils.DUMMY_CONTAINER):
       if isinstance(other_type, abstract.TypeParameter):
+        self._seen_type_params.add(other_type)
         new_subst = self._match_type_param_against_type_param(
             left.param, other_type, subst, view)
         if new_subst is not None:
@@ -424,6 +539,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         # left to its upper bound.
         return self._instantiate_and_match(left.param, other_type, subst, view)
     elif isinstance(other_type, abstract.TypeParameter):
+      self._seen_type_params.add(other_type)
       for c in other_type.constraints:
         new_subst = self._match_value_against_type(value, c, subst, view)
         if new_subst is not None:
@@ -441,58 +557,8 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
                            self._node, abstract_utils.DUMMY_CONTAINER)}
           self._error_subst = self._merge_substs(subst, [new_subst])
           return None
-      if other_type.full_name in subst:
-        # Merge the two variables.
-        new_var = subst[other_type.full_name].AssignToNewVariable(self._node)
-        new_var.AddBinding(left, [], self._node)
-      else:
-        new_left = self.ctx.convert.get_maybe_abstract_instance(left)
-        new_var = self.ctx.program.NewVariable()
-        new_var.AddBinding(new_left, {value}, self._node)
-
-      type_key = left.get_type_key()
-      # Every value with this type key produces the same result when matched
-      # against other_type, so they can all be added to this substitution rather
-      # than matched separately.
-      for other_value in value.variable.bindings:
-        if (other_value is not value and
-            other_value.data.get_type_key() == type_key):
-          new_var.AddBinding(other_value.data, {other_value}, self._node)
-      if other_type.constraints:
-        new_values = self._discard_ambiguous_values(new_var.data)
-        has_error = not self._satisfies_single_type(new_values)
-        if not has_error and new_values and len(new_values) < len(new_var.data):
-          # We can filter out ambiguous values because we've already found the
-          # single concrete type allowed for this variable.
-          new_var = self.ctx.program.NewVariable(new_values, [], self._node)
-      else:
-        if other_type.full_name in subst:
-          has_error = False
-          old_values = subst[other_type.full_name].data
-          # If _discard_ambiguous_values does not discard 'left', then it is a
-          # concrete value that we need to match.
-          if old_values and self._discard_ambiguous_values([left]):
-            old_concrete_values = self._discard_ambiguous_values(old_values)
-            # If any of the previous TypeVar values were ambiguous, then we
-            # treat the match as a success. Otherwise, 'left' needs to match at
-            # least one of them.
-            if len(old_values) == len(old_concrete_values):
-              has_error = True
-              for old_value in old_concrete_values:
-                if self._satisfies_common_superclass([left, old_value]):
-                  has_error = False
-                elif old_value.cls.is_protocol:
-                  with self._track_partially_matched_protocols():
-                    new_subst = self._match_against_protocol(
-                        left, old_value.cls, subst, view)
-                    if new_subst is not None:
-                      has_error = False
-                      subst = new_subst
-                if not has_error:
-                  break
-        else:
-          has_error = not self._satisfies_common_superclass(
-              self._discard_ambiguous_values(new_var.data))
+      new_var, has_error = self._check_type_param_consistency(
+          left, value, other_type, subst)
       if has_error:
         self._error_subst = subst
         return None
@@ -553,6 +619,61 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       log.error("Invalid type: %s", type(other_type))
       return None
 
+  def _check_type_param_consistency(
+      self, new_value: abstract.BaseValue, new_value_binding: cfg.Binding,
+      t: abstract.TypeParameter, subst: _SubstType
+  ) -> Tuple[cfg.Variable, bool]:
+    if t.full_name in subst:
+      # Merge the two variables.
+      new_var = subst[t.full_name].AssignToNewVariable(self._node)
+      new_var.AddBinding(new_value, [], self._node)
+    else:
+      new_var = self.ctx.program.NewVariable()
+      new_var.AddBinding(
+          self.ctx.convert.get_maybe_abstract_instance(new_value),
+          {new_value_binding}, self._node)
+    type_key = new_value.get_type_key()
+    # Every value with this type key produces the same result when matched
+    # against t, so they can all be added to this substitution rather than
+    # matched separately.
+    for other_value in new_value_binding.variable.bindings:
+      if (other_value is not new_value_binding and
+          other_value.data.get_type_key() == type_key):
+        new_var.AddBinding(other_value.data, {other_value}, self._node)
+    if t.constraints:
+      new_values = self._discard_ambiguous_values(new_var.data)
+      has_error = not self._satisfies_single_type(new_values)
+      if not has_error and new_values and len(new_values) < len(new_var.data):
+        # We can filter out ambiguous values because we've already found the
+        # single concrete type allowed for this variable.
+        new_var = self.ctx.program.NewVariable(new_values, [], self._node)
+    else:
+      if t.full_name in subst:
+        has_error = False
+        old_values = subst[t.full_name].data
+        # If _discard_ambiguous_values does not discard 'new_value', then it is
+        # a concrete value that we need to match.
+        if old_values and self._discard_ambiguous_values([new_value]):
+          old_concrete_values = self._discard_ambiguous_values(old_values)
+          # If any of the previous TypeVar values were ambiguous, then we
+          # treat the match as a success. Otherwise, 'new_value' needs to match
+          # at least one of them.
+          if len(old_values) == len(old_concrete_values):
+            has_error = True
+            for old_value in old_concrete_values:
+              if self._satisfies_common_superclass([new_value, old_value]):
+                has_error = False
+              elif old_value.cls.is_protocol:
+                with self._track_partially_matched_protocols():
+                  has_error = self._match_against_protocol(
+                      new_value, old_value.cls, subst, {}) is None
+              if not has_error:
+                break
+      else:
+        has_error = not self._satisfies_common_superclass(
+            self._discard_ambiguous_values(new_var.data))
+    return new_var, has_error
+
   def _match_type_against_type(self, left, other_type, subst, view):
     """Checks whether a type is compatible with a (formal) type.
 
@@ -699,6 +820,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     return self._merge_substs(subst, new_substs)
 
   def _mutate_type_parameters(self, params, value, subst):
+    self._seen_type_params.update(params)
     new_subst = {p.full_name: value.to_variable(self._node) for p in params}
     return self._merge_substs(subst, [new_subst])
 
@@ -740,6 +862,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
           not isinstance(right, abstract.TypeParameter) or
           right.constraints or right.bound or callable_param_count[right] != 1):
         return None
+      self._seen_type_params.add(right)
       subst = subst.copy()
       # We don't know what to fill in here, since we have a TypeVar matching a
       # TypeVar, but we have to add *something* to indicate the match succeeded.
@@ -796,6 +919,70 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
           subst[name].PasteVariable(var)
     return subst
 
+  def _merge_matches(
+      self, name: str, formal: abstract.BaseValue,
+      old_matches: Optional[List[GoodMatch]], new_matches: List[GoodMatch],
+      has_self: bool) -> List[GoodMatch]:
+    if old_matches is None:
+      return new_matches
+    combined_matches = []
+    for new_match in new_matches:
+      cur_types = datatypes.AliasingDict(
+          {t.full_name: t for t in self._seen_type_params},
+          aliases=new_match.subst.aliases)
+      matched = False
+      bad_param = None
+      for old_match in old_matches:
+        combined_subst = self._match_subst_against_subst(
+            old_match.subst, new_match.subst, cur_types, has_self)
+        if combined_subst is None:
+          if not bad_param:
+            self._error_subst = old_match.subst
+            bad_param = self._get_bad_type(name, formal)
+          continue
+        combined_matches.append(
+            GoodMatch.merge(old_match, new_match, combined_subst))
+        matched = True
+      if not matched:
+        raise self.MatchError(bad_param)
+    return combined_matches
+
+  def _match_subst_against_subst(
+      self, old_subst, new_subst, type_param_map, has_self):
+    subst = datatypes.AliasingDict(aliases=old_subst.aliases)
+    for t in new_subst:
+      if t not in old_subst or not old_subst[t].bindings:
+        subst[t] = new_subst[t]
+        continue
+      if not new_subst[t].bindings:
+        subst[t] = old_subst[t]
+        continue
+      # Any type parameters in old subst that were matched from a 'self' arg are
+      # class parameters whose values have been declared by the user, e.g.:
+      #   x = Container[int](__any_object__)
+      # We should keep the 'int' value rather than using Union[int, Unknown].
+      # Note that we still need to check that the new values are consistent with
+      # the old ones.
+      keep_old_values = has_self and any(not isinstance(v, abstract.Empty)
+                                         for v in old_subst[t].data)
+      for b1 in old_subst[t].bindings:
+        for b2 in new_subst[t].bindings:
+          new_var, has_error = self._check_type_param_consistency(
+              b2.data, b2, type_param_map[t],
+              old_subst.copy(t=b1.AssignToNewVariable(self._node)))
+          if has_error:
+            continue
+          if t not in subst:
+            subst[t] = old_subst[t] if keep_old_values else new_var
+          elif not keep_old_values:
+            subst[t].PasteVariable(new_var, self._node)
+      if t not in subst:
+        return None
+    for t in old_subst:
+      if t not in subst:
+        subst[t] = old_subst[t]
+    return subst
+
   def _instantiate_and_match(self, left, other_type, subst, view,
                              container=None):
     """Instantiate and match an abstract value."""
@@ -1052,7 +1239,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       if k not in fields:
         continue
       typ = abstract_utils.get_atomic_value(fields[k])
-      match_result = self.compute_matches(v, typ)
+      match_result = self.compute_one_match(v, typ)
       if not match_result.success:
         bad.append((k, match_result.bad_matches))
     if missing or extra or bad:
@@ -1089,8 +1276,15 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     """
     if isinstance(left.cls, abstract.AMBIGUOUS_OR_EMPTY):
       return subst
-    elif left.cls.is_dynamic:
+    elif left.cls.is_dynamic:  # pytype: disable=attribute-error
       return self._subst_with_type_parameters_from(subst, other_type)
+    elif (self.ctx.options.mapping_is_not_sequence and
+          other_type.full_name == "typing.Sequence" and
+          any(cls.full_name == "typing.Mapping" for cls in left.cls.mro)):
+      # A mapping should not be considered a sequence, even though pytype says
+      # that dict[int | slice, str] satisfies the Sequence[str] protocol:
+      # https://docs.python.org/3/c-api/sequence.html#c.PySequence_Check
+      return None
     left_attributes = self._get_attribute_names(left)
     missing = other_type.protocol_attributes - left_attributes
     if missing:  # not all protocol attributes are implemented by 'left'
@@ -1172,10 +1366,11 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         callable_signature.formal_type_parameters[0] = (
             self.ctx.convert.unsolvable)
       if isinstance(other_type, abstract.ParameterizedClass):
-        annotation_subst = datatypes.AliasingDict()
         if isinstance(other_type.base_cls, abstract.Class):
-          annotation_subst.uf = (
-              other_type.base_cls.all_formal_type_parameters.uf)
+          aliases = other_type.base_cls.all_formal_type_parameters.aliases
+        else:
+          aliases = None
+        annotation_subst = datatypes.AliasingDict(aliases=aliases)
         for (param, value) in other_type.get_formal_type_parameters().items():
           annotation_subst[param] = value.instantiate(
               self._node, abstract_utils.DUMMY_CONTAINER)
@@ -1318,6 +1513,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
   def _subst_with_type_parameters_from(self, subst, typ):
     subst = subst.copy()
     for param in self.ctx.annotation_utils.get_type_parameters(typ):
+      self._seen_type_params.add(param)
       if param.full_name not in subst:
         subst[param.full_name] = self.ctx.convert.empty.to_variable(self._node)
     return subst
