@@ -2,6 +2,7 @@
 
 import collections
 import copy
+import keyword
 import logging
 import re
 
@@ -284,6 +285,17 @@ class PrintVisitor(base_visitor.Visitor):
   def VisitClass(self, node):
     """Visit a class, producing a multi-line, properly indented string."""
     bases = node.bases
+    if bases == ("TypedDict",):
+      constants = {}
+      for c in node.constants:
+        name, typ = c.split(": ")
+        constants[name] = typ
+      if any(keyword.iskeyword(name) for name in constants):
+        # We output the TypedDict in functional form, since using the class form
+        # would produce a parse error when the pyi file is ingested.
+        fields = "{%s}" % ", ".join(f"{name!r}: {typ}"
+                                    for name, typ in constants.items())
+        return f"{node.name} = TypedDict('{node.name}', {fields})"
     # If object is the only base, we don't need to list any bases.
     if bases == ("object",):
       bases = ()
